@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2021, 2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -95,7 +95,8 @@ class Saturation:
                     radicals[n] = r
                     for m in env:
                         if m not in seen:
-                            bonds[n][m] = bonds[m][n] = Bond(1)
+                            bonds[n][m] = bonds[m][n] = b = Bond(1)
+                            b._attach_graph(self, n, m)
                 else:
                     unsaturated[n] = [(c, r, h)]
             else:
@@ -141,7 +142,8 @@ class Saturation:
                     return False
 
             for n, m, b in sb:
-                bonds[n][m] = bonds[m][n] = Bond(b)
+                bonds[n][m] = bonds[m][n] = b = Bond(b)
+                b._attach_graph(self, n, m)
             for n, c, r in sa:
                 charges[n] = c
                 radicals[n] = r
@@ -225,7 +227,8 @@ def _saturate(bonds, atoms, expected_radicals_count, expected_charge):
             if not env:
                 es = [(n, c, r) for c, r, h in atoms[n] if not h]
                 if not es:
-                    return  # saturation impossible
+                    raise ValenceError('Saturation impossible. '
+                                       f"Isolated atom ({n}) doesn't have appropriate charge-radical state")
                 to_del.append(n)
                 dots.append(es)
         for n in to_del:
